@@ -19,6 +19,7 @@
  */
 
 #include <libgda/libgda.h>
+#include <glib/gi18n-lib.h>
 #include "gdaui-entry-combo.h"
 #include "gdaui-combo.h"
 #include "gdaui-data-store.h"
@@ -54,6 +55,8 @@ static void            gdaui_entry_combo_set_attributes    (GdauiDataEntry *de,
 static GdaValueAttribute gdaui_entry_combo_get_attributes    (GdauiDataEntry *de);
 static gboolean        gdaui_entry_combo_can_expand  (GdauiDataEntry *de, gboolean horiz);
 static void            gdaui_entry_combo_grab_focus        (GdauiDataEntry *de);
+static void            gdaui_entry_combo_set_unknown_color (GdauiDataEntry *de, gdouble red, gdouble green,
+							    gdouble blue, gdouble alpha);
 
 static void           _gdaui_entry_combo_construct(GdauiEntryCombo* combo, 
 						   GdauiSet *paramlist, GdauiSetSource *source);
@@ -68,10 +71,10 @@ enum {
  * used.
  */
 typedef struct {
-	GdaSetNode *node;
-	const GValue       *value;    /* we don't own the value, since it belongs to a GdaDataModel => don't free it */
-	GValue             *value_orig;
-	GValue             *value_default;
+	GdaSetNode   *node;
+	GValue       *value;
+	GValue       *value_orig;
+	GValue       *value_default;
 } ComboNode;
 #define COMBO_NODE(x) ((ComboNode*)(x))
 
@@ -142,6 +145,7 @@ gdaui_entry_combo_data_entry_init (GdauiDataEntryIface *iface)
         iface->get_handler = NULL;
         iface->can_expand = gdaui_entry_combo_can_expand;
 	iface->grab_focus = gdaui_entry_combo_grab_focus;
+	iface->set_unknown_color = gdaui_entry_combo_set_unknown_color;
 }
 
 
@@ -160,12 +164,6 @@ gdaui_entry_combo_class_init (GdauiEntryComboClass *class)
         g_object_class_install_property (object_class, PROP_SET_DEFAULT_IF_INVALID,
 					 g_param_spec_boolean ("set-default-if-invalid", NULL, NULL, FALSE,
                                                                (G_PARAM_READABLE | G_PARAM_WRITABLE)));
-
-	/* RC setting */
-#define RC_STRING						\
-	"style \"gnomedb\" { GdauiCombo::appears-as-list = 1 }" \
-		"class \"GtkComboBox\" style \"gnomedb\""
-	/*gtk_rc_parse_string (RC_STRING);*/
 }
 
 static void
@@ -294,10 +292,10 @@ void _gdaui_entry_combo_construct (GdauiEntryCombo* combo, GdauiSet *paramlist,
 		ComboNode *cnode = g_new0 (ComboNode, 1);
 		
 		cnode->node = GDA_SET_NODE (list->data);
-		cnode->value = gda_holder_get_value (cnode->node->holder);
+		cnode->value = NULL;
 		combo->priv->combo_nodes = g_slist_append (combo->priv->combo_nodes, cnode);
 
-		values = g_slist_append (values, (GValue *) cnode->value);
+		values = g_slist_append (values, (GValue *) gda_holder_get_value (cnode->node->holder));
 		if (gda_holder_get_not_null (cnode->node->holder))
 			null_possible = FALSE;
 	}
@@ -308,18 +306,24 @@ void _gdaui_entry_combo_construct (GdauiEntryCombo* combo, GdauiSet *paramlist,
 					    combo->priv->source->shown_n_cols, 
 					    combo->priv->source->shown_cols_index);
 	g_object_set (G_OBJECT (entry), "as-list", TRUE, NULL);
-	g_signal_connect (G_OBJECT (entry), "changed",
-			  G_CALLBACK (combo_contents_changed_cb), combo);
 
 	gdaui_entry_shell_pack_entry (GDAUI_ENTRY_SHELL (combo), entry);
 	gtk_widget_show (entry);
 	combo->priv->combo_entry = entry;
 
-	_gdaui_combo_set_selected_ext (GDAUI_COMBO (entry), values, NULL);
-	g_slist_free (values);
+	if (values) {
+		_gdaui_combo_set_selected_ext (GDAUI_COMBO (entry), values, NULL);
+		gdaui_entry_combo_set_reference_values (combo, values);
+		g_slist_free (values);
+	}
+
 	gdaui_combo_add_null (GDAUI_COMBO (entry), combo->priv->null_possible);
 
 	combo->priv->data_valid = combo->priv->null_possible ? TRUE : FALSE;
+
+	combo_contents_changed_cb ((GdauiCombo*) entry, combo);
+	g_signal_connect (G_OBJECT (entry), "changed",
+			  G_CALLBACK (combo_contents_changed_cb), combo);
 }
 
 static void
@@ -355,12 +359,9 @@ gdaui_entry_combo_dispose (GObject *object)
 			for (list = combo->priv->combo_nodes; list; list = list->next) {
 				ComboNode *node = COMBO_NODE (list->data);
 
-				if (node->value)
-					node->value = NULL; /* don't free that value since we have not copied it */
-				if (node->value_orig)
-					gda_value_free (node->value_orig);
-				if (node->value_default)
-					gda_value_free (node->value_default);
+				gda_value_free (node->value);
+				gda_value_free (node->value_orig);
+				gda_value_free (node->value_default);
 				g_free (node);
 			}
 			g_slist_free (combo->priv->combo_nodes);
@@ -444,9 +445,12 @@ combo_contents_changed_cb (G_GNUC_UNUSED GdauiCombo *entry, GdauiEntryCombo *com
 		for (list = combo->priv->combo_nodes; list; list = list->next) {
 			ComboNode *node = COMBO_NODE (list->data);
 
+			gda_value_free (node->value);
 			gtk_tree_model_get (model, &iter, node->node->source_column, &(node->value), -1);
-			/*g_print ("%s(): Set Combo Node value to %s\n", __FUNCTION__,
-			  node->value ? gda_value_stringify (node->value) : "Null");*/
+			if (node->value)
+				node->value = gda_value_copy (node->value);
+			/*g_print ("%s (%p): Set Combo Node value to %s (%p)\n", __FUNCTION__, combo, 
+			  node->value ? gda_value_stringify (node->value) : "Null", node->value);*/
 		}
 		
 		g_signal_emit_by_name (G_OBJECT (combo), "status-changed");
@@ -457,7 +461,7 @@ combo_contents_changed_cb (G_GNUC_UNUSED GdauiCombo *entry, GdauiEntryCombo *com
 /**
  * gdaui_entry_combo_set_values:
  * @combo: a #GdauiEntryCombo widet
- * @values: (element-type GValue): a list of #GValue values, or %NULL
+ * @values: (element-type GValue) (allow-none): a list of #GValue values, or %NULL
  *
  * Sets the values of @combo to the specified ones. None of the
  * values provided in the list is modified.
@@ -510,8 +514,11 @@ gdaui_entry_combo_set_values (GdauiEntryCombo *combo, GSList *values)
 			/* adjusting the values */
 			for (list = combo->priv->combo_nodes; list; list = list->next) {
 				node = COMBO_NODE (list->data);
+				gda_value_free (node->value);
 				gtk_tree_model_get (model, &iter, node->node->source_column,
 						    &(node->value), -1);
+				if (node->value)
+					node->value = gda_value_copy (node->value);
 			}
 			
 			combo->priv->null_forced = FALSE;
@@ -523,8 +530,10 @@ gdaui_entry_combo_set_values (GdauiEntryCombo *combo, GSList *values)
 	}
 	else  { /* set to NULL */
 		GSList *list;
-		for (list = combo->priv->combo_nodes; list; list = list->next)
+		for (list = combo->priv->combo_nodes; list; list = list->next) {
+			gda_value_free (COMBO_NODE (list->data)->value);
 			COMBO_NODE (list->data)->value = NULL;
+		}
 
 		if (combo->priv->null_possible) {
 			real_combo_block_signals (combo);
@@ -541,9 +550,8 @@ gdaui_entry_combo_set_values (GdauiEntryCombo *combo, GSList *values)
 	combo->priv->data_valid = !err;
 	g_signal_emit_by_name (G_OBJECT (combo), "status-changed");
 
-	if (!err) 
-		/* notify the status and contents changed */
-		gdaui_entry_combo_emit_signal (combo);
+	/* notify the status and contents changed */
+	gdaui_entry_combo_emit_signal (combo);
 
 	return !err;
 }
@@ -1047,3 +1055,10 @@ gdaui_entry_combo_grab_focus (GdauiDataEntry *iface)
 	if (combo->priv->combo_entry)
 		gtk_widget_grab_focus (combo->priv->combo_entry);
 }
+
+static void
+gdaui_entry_combo_set_unknown_color (GdauiDataEntry *de, gdouble red, gdouble green,
+				     gdouble blue, gdouble alpha)
+{
+	gdaui_entry_shell_set_ucolor (GDAUI_ENTRY_SHELL (de), red, green, blue, alpha);
+}
