@@ -147,7 +147,8 @@ gdaui_entry_common_time_class_init (GdauiEntryCommonTimeClass * class)
 	object_class->get_property = gdaui_entry_common_time_get_property;
 
 	g_object_class_install_property (object_class, PROP_EDITING_CANCELED,
-					 g_param_spec_boolean ("editing-canceled", NULL, NULL, FALSE, G_PARAM_READABLE));
+					 g_param_spec_boolean ("editing-canceled", NULL, NULL, FALSE,
+							       G_PARAM_READABLE | G_PARAM_WRITABLE));
 	g_object_class_install_property (object_class, PROP_TYPE,
 					 g_param_spec_uint ("type", NULL, NULL, 0, G_MAXUINT, GDA_TYPE_TIME, 
 							    G_PARAM_WRITABLE | G_PARAM_READABLE));
@@ -156,7 +157,7 @@ gdaui_entry_common_time_class_init (GdauiEntryCommonTimeClass * class)
 static gboolean
 key_press_event_cb (GdauiEntryCommonTime *mgtim, GdkEventKey *key_event, G_GNUC_UNUSED gpointer data)
 {
-	if (key_event->keyval == GDK_Escape)
+	if (key_event->keyval == GDK_KEY_Escape)
 		mgtim->priv->editing_canceled = TRUE;
 	return FALSE;
 }
@@ -193,7 +194,6 @@ gdaui_entry_common_time_new (GdaDataHandler *dh, GType type)
 	GdauiEntryCommonTime *mgtim;
 
 	g_return_val_if_fail (dh && GDA_IS_DATA_HANDLER (dh), NULL);
-	g_return_val_if_fail (type != G_TYPE_INVALID, NULL);
 	g_return_val_if_fail (gda_data_handler_accepts_g_type (dh, type), NULL);
 
 	obj = g_object_new (GDAUI_TYPE_ENTRY_COMMON_TIME, "handler", dh, NULL);
@@ -259,6 +259,9 @@ gdaui_entry_common_time_set_property (GObject *object,
 		case PROP_TYPE:
 			gdaui_data_entry_set_value_type (GDAUI_DATA_ENTRY (object), g_value_get_uint (value));
 			break;
+		case PROP_EDITING_CANCELED:
+			TO_IMPLEMENT;
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
@@ -560,7 +563,7 @@ create_entry_date (GdauiEntryCommonTime *mgtim)
 	GdaDataHandler *dh;
 
 	/* top widget */
-	hb = gtk_hbox_new (FALSE, 3);
+	hb = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 3);
 
 	/* text entry */
 	dh = gdaui_data_entry_get_handler (GDAUI_DATA_ENTRY (mgtim));
@@ -728,7 +731,7 @@ date_delete_popup (G_GNUC_UNUSED GtkWidget *widget, GdauiEntryCommonTime *mgtim)
 static gint
 date_key_press_popup (GtkWidget *widget, GdkEventKey *event, GdauiEntryCommonTime *mgtim)
 {
-	if (event->keyval != GDK_Escape)
+	if (event->keyval != GDK_KEY_Escape)
                 return FALSE;
 
         g_signal_stop_emission_by_name (widget, "key-press-event");
@@ -797,7 +800,7 @@ date_day_selected_double_click (G_GNUC_UNUSED GtkCalendar *calendar, GdauiEntryC
 }
 
 
-static gboolean popup_grab_on_window (GdkWindow *window, guint32 activate_time);
+static gboolean popup_grab_on_window (GtkWidget *widget, guint32 activate_time);
 static void position_popup (GdauiEntryCommonTime *mgtim);
 static void
 date_calendar_choose_cb (GtkWidget *button, GdauiEntryCommonTime *mgtim)
@@ -867,17 +870,12 @@ date_calendar_choose_cb (GtkWidget *button, GdauiEntryCommonTime *mgtim)
 
         /* popup window */
         /* Temporarily grab pointer and keyboard, copied from GnomeDateEdit */
-#if GTK_CHECK_VERSION(2,18,0)
-        if (!popup_grab_on_window (gtk_widget_get_window (button), gtk_get_current_event_time ()))
+        if (!popup_grab_on_window (button, gtk_get_current_event_time ()))
                 return;
-#else
-	if (!popup_grab_on_window (button->window, gtk_get_current_event_time ()))
-		return;
-#endif
 
         position_popup (mgtim);
-        gtk_grab_add (mgtim->priv->window);
         gtk_widget_show (mgtim->priv->window);
+        gtk_grab_add (mgtim->priv->window);
 
 	GdkScreen *screen;
 	gint swidth, sheight;
@@ -915,27 +913,31 @@ date_calendar_choose_cb (GtkWidget *button, GdauiEntryCommonTime *mgtim)
 		gtk_window_move (GTK_WINDOW (mgtim->priv->window), root_x, root_y);
 
         gtk_widget_grab_focus (mgtim->priv->date);
-#if GTK_CHECK_VERSION(2,18,0)
-        popup_grab_on_window (gtk_widget_get_window (mgtim->priv->window),
+        popup_grab_on_window (mgtim->priv->window,
                               gtk_get_current_event_time ());
-#else
-	popup_grab_on_window (mgtim->priv->window->window,
-			      gtk_get_current_event_time ());
-#endif
 }
 
 static gboolean
-popup_grab_on_window (GdkWindow *window, guint32 activate_time)
+popup_grab_on_window (GtkWidget *widget, guint32 activate_time)
 {
-        if ((gdk_pointer_grab (window, TRUE,
-                               GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
-                               GDK_POINTER_MOTION_MASK,
-                               NULL, NULL, activate_time) == 0)) {
-                if (gdk_keyboard_grab (window, TRUE,
-                                       activate_time) == 0)
+	GdkDeviceManager *manager;
+	GdkDevice *pointer;
+	GdkWindow *window;
+	window = gtk_widget_get_window (widget);
+	manager = gdk_display_get_device_manager (gtk_widget_get_display (widget));
+	pointer = gdk_device_manager_get_client_pointer (manager);
+        if (gdk_device_grab (pointer, window, GDK_OWNERSHIP_WINDOW, TRUE,
+			     GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
+			     GDK_POINTER_MOTION_MASK,
+			     NULL, activate_time) == GDK_GRAB_SUCCESS) {
+		GdkDevice *keyb;
+		keyb = gdk_device_get_associated_device (pointer);
+                if (gdk_device_grab (keyb, window, GDK_OWNERSHIP_WINDOW, TRUE,
+				     GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK, NULL, activate_time) ==
+		    GDK_GRAB_SUCCESS)
                         return TRUE;
                 else {
-                        gdk_pointer_ungrab (activate_time);
+                        gdk_device_ungrab (pointer, activate_time);
                         return FALSE;
                 }
         }
@@ -947,11 +949,11 @@ position_popup (GdauiEntryCommonTime *mgtim)
 {
         gint x, y;
         gint bwidth, bheight;
-        GtkRequisition req;
+        GtkRequisition req_minimum, req_natural;
 
-        gtk_widget_size_request (mgtim->priv->window, &req);
+        gtk_widget_get_preferred_size (mgtim->priv->window, &req_minimum,
+				       &req_natural);
 
-#if GTK_CHECK_VERSION(2,18,0)
         gdk_window_get_origin (gtk_widget_get_window (mgtim->priv->date_button), &x, &y);
 	GtkAllocation alloc;
 	gtk_widget_get_allocation (mgtim->priv->date_button, &alloc);
@@ -959,16 +961,8 @@ position_popup (GdauiEntryCommonTime *mgtim)
         y += alloc.y;
         bwidth = alloc.width;
         bheight = alloc.height;
-#else
-	gdk_window_get_origin (mgtim->priv->date_button->window, &x, &y);
-	
-        x += mgtim->priv->date_button->allocation.x;
-        y += mgtim->priv->date_button->allocation.y;
-        bwidth = mgtim->priv->date_button->allocation.width;
-	bheight = mgtim->priv->date_button->allocation.height;
-#endif
 
-        x += bwidth - req.width;
+        x += bwidth - req_natural.width;
         y += bheight;
 
         if (x < 0)
@@ -1094,7 +1088,7 @@ create_entry_ts (GdauiEntryCommonTime *mgtim)
 {
 	GtkWidget *hb, *wid;
 
-	hb = gtk_hbox_new (FALSE, 0);
+	hb = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 	
 	/* date part */
 	wid = create_entry_date (mgtim);
