@@ -1,20 +1,21 @@
 /*
- * Copyright (C) 2005 - 2011 Vivien Malerba
+ * Copyright (C) 2006 - 2007 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2006 - 2011 Vivien Malerba <malerba@gnome-db.org>
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 
@@ -111,11 +112,23 @@ struct _GdaDataModelIterClass
  * Note: the new #GdaDataModelIter does not hold any reference to the data model it iterates through (ie.
  * if this data model is destroyed at some point, the new iterator will become useless but in
  * any case it will not prevent the data model from being destroyed).
+ *
+ * Note: when the data model emits the "reset" signal, then:
+ * <itemizedlist>
+ *  <listitem><para>the number of columns of the iterator can change to reflect the new data model
+ *    being itered on. In this case the iterator's position is reset as if it was
+ *    just created</para></listitem>
+ *  <listitem><para>some column types which were unknown (i.e. GDA_TYPE_NULL type), can change
+ *    to their correct type. In this case there is no other iterator change</para></listitem>
+ *  <listitem><para>some column types which were not GDA_TYPE_NULL can change, and in this case
+ *    the iterator's position is reset as if it was just created</para></listitem>
+ * </itemizedlist>
  */
 
 GType             gda_data_model_iter_get_type             (void) G_GNUC_CONST;
 
 const GValue     *gda_data_model_iter_get_value_at         (GdaDataModelIter *iter, gint col);
+const GValue     *gda_data_model_iter_get_value_at_e       (GdaDataModelIter *iter, gint col, GError **error);
 const GValue     *gda_data_model_iter_get_value_for_field  (GdaDataModelIter *iter, const gchar *field_name);
 gboolean          gda_data_model_iter_set_value_at         (GdaDataModelIter *iter, gint col, 
 							    const GValue *value, GError **error);
