@@ -132,7 +132,7 @@ gda_data_model_class_init (G_GNUC_UNUSED gpointer g_class)
 	MUTEX_LOCK();
 	if (! initialized) {
 		/**
-		 * GdaDataModel::changed:
+		 * GdaDataModel::changed
 		 * @model: the #GdaDataModel
 		 *
 		 * Gets emitted when any value in @model has been changed
@@ -146,7 +146,7 @@ gda_data_model_class_init (G_GNUC_UNUSED gpointer g_class)
 				      g_cclosure_marshal_VOID__VOID,
 				      G_TYPE_NONE, 0);
 		/**
-		 * GdaDataModel::row-inserted:
+		 * GdaDataModel::row-inserted
 		 * @model: the #GdaDataModel
 		 * @row: the row number
 		 *
@@ -161,7 +161,7 @@ gda_data_model_class_init (G_GNUC_UNUSED gpointer g_class)
 				      g_cclosure_marshal_VOID__INT,
 				      G_TYPE_NONE, 1, G_TYPE_INT);
 		/**
-		 * GdaDataModel::row-updated:
+		 * GdaDataModel::row-updated
 		 * @model: the #GdaDataModel
 		 * @row: the row number
 		 *
@@ -176,7 +176,7 @@ gda_data_model_class_init (G_GNUC_UNUSED gpointer g_class)
 				      g_cclosure_marshal_VOID__INT,
 				      G_TYPE_NONE, 1, G_TYPE_INT);
 		/**
-		 * GdaDataModel::row-removed:
+		 * GdaDataModel::row-removed
 		 * @model: the #GdaDataModel
 		 * @row: the row number
 		 *
@@ -191,7 +191,7 @@ gda_data_model_class_init (G_GNUC_UNUSED gpointer g_class)
 				      g_cclosure_marshal_VOID__INT,
 				      G_TYPE_NONE, 1, G_TYPE_INT);
 		/**
-		 * GdaDataModel::reset:
+		 * GdaDataModel::reset
 		 * @model: the #GdaDataModel
 		 *
 		 * Gets emitted when @model's contents has been completely reset (the number and
@@ -207,7 +207,7 @@ gda_data_model_class_init (G_GNUC_UNUSED gpointer g_class)
 				      G_TYPE_NONE, 0);
 
 		/**
-		 * GdaDataModel::access-changed:
+		 * GdaDataModel::access-changed
 		 * @model: the #GdaDataModel
 		 *
 		 * Gets emitted when @model's access flags have changed. Use
@@ -369,7 +369,6 @@ gda_data_model_reset (GdaDataModel *model)
  * re-enable notifications again, you should call the
  * #gda_data_model_thaw function.
  *
- * Virtual: i_set_notify
  */
 void
 gda_data_model_freeze (GdaDataModel *model)
@@ -395,6 +394,44 @@ gda_data_model_thaw (GdaDataModel *model)
 		(GDA_DATA_MODEL_GET_CLASS (model)->i_set_notify) (model, TRUE);
 }
 
+/**
+ * gda_data_model_set_notify:
+ * @model: a #GdaDataModel object.
+ * @do_notify_changes: Set to TRUE if you require notifications.
+ *
+ * Enable or disable notifications changes on the given data model.
+ * 
+ * Deprecated: 5.2: use gda_data_model_freeze() and gda_data_model_thaw() instead
+ *
+ * Virtual: i_set_notify
+ */
+void
+gda_data_model_set_notify (GdaDataModel *model, gboolean do_notify_changes)
+{
+	g_return_if_fail (GDA_IS_DATA_MODEL (model));
+	if (GDA_DATA_MODEL_GET_CLASS (model)->i_set_notify)
+		(GDA_DATA_MODEL_GET_CLASS (model)->i_set_notify) (model, do_notify_changes);
+}
+
+/**
+ * gda_data_model_get_notify:
+ * @model: a #GdaDataModel object.
+ *
+ * Returns the status of notifications changes on the given data model.
+ * 
+ * Virtual: i_get_notify
+ */
+gboolean
+gda_data_model_get_notify (GdaDataModel *model)
+{
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+	if (GDA_DATA_MODEL_GET_CLASS (model)->i_get_notify)
+		return (GDA_DATA_MODEL_GET_CLASS (model)->i_get_notify) (model);
+	else
+		return TRUE;
+}
+
+
 /**
  * gda_data_model_get_access_flags:
  * @model: a #GdaDataModel object.
@@ -790,7 +827,7 @@ gda_data_model_set_value_at (GdaDataModel *model, gint col, gint row, const GVal
  * gda_data_model_set_values:
  * @model: a #GdaDataModel object.
  * @row: row number.
- * @values: (element-type GObject.Value) (transfer none) (allow-none): a list of #GValue (or %NULL), one for at most the number of columns of @model
+ * @values: (element-type GLib.Value) (transfer none) (allow-none): a list of #GValue (or %NULL), one for at most the number of columns of @model
  * @error: a place to store errors, or %NULL
  *
  * In a similar way to gda_data_model_set_value_at(), this method modifies a data model's contents
@@ -888,10 +925,97 @@ gda_data_model_create_iter (GdaDataModel *model)
 				      "data-model", model, NULL);
 }
 
+/**
+ * gda_data_model_iter_at_row:
+ * @model: a #GdaDataModel object.
+ * @iter: a #GdaDataModelIter object.
+ * @row: a row to point to with @iter
+ *
+ * Moves @iter to the row number given by @row.
+ * 
+ * Deprecated: 5.2: use gda_data_model_iter_move_to_row() instead
+ *
+ * Virtual: i_iter_at_row
+ */
+gboolean
+gda_data_model_iter_at_row (GdaDataModel *model, GdaDataModelIter *iter, gint row)
+{
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (model), FALSE);
+
+	return gda_data_model_iter_move_to_row (iter, row);
+}
+
+/**
+ * gda_data_model_iter_next:
+ * @model: a #GdaDataModel object.
+ * @iter: a #GdaDataModelIter object.
+ *
+ * Moves @iter to the next row in @model.
+ * 
+ * Deprecated: 5.2: use gda_data_model_iter_move_next() instead
+ *
+ * Virtual: i_iter_next
+ */
+gboolean
+gda_data_model_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
+{
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (model), FALSE);
+
+	return gda_data_model_iter_move_next (iter);
+}
+
+/**
+ * gda_data_model_iter_prev:
+ * @model: a #GdaDataModel object.
+ * @iter: a #GdaDataModelIter object.
+ *
+ * Moves @iter to the next row in @model.
+ *
+ * Deprecated: 5.2: use gda_data_model_iter_move_prev() instead
+ * 
+ * Virtual: i_iter_prev
+ */
+gboolean 
+gda_data_model_iter_prev (GdaDataModel *model, GdaDataModelIter *iter)
+{
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (model), FALSE);
+
+	return gda_data_model_iter_move_prev (iter);
+}
+
+
+/**
+ * gda_data_model_iter_set_value:
+ * @model: a #GdaDataModel object.
+ * @iter: a #GdaDataModelIter object.
+ * @col: the number of column to set value to
+ * @value: the to use to set on
+ * @error: a place to set errors
+ *
+ * Set @value to the given @column and row pointed by @iter in the given @model.
+ * 
+ * Deprecated: 5.2: use gda_data_model_iter_set_value_at() instead
+ *
+ * Virtual: i_iter_set_value
+ */
+gboolean 
+gda_data_model_iter_set_value (GdaDataModel *model, GdaDataModelIter *iter, gint col,
+			       const GValue *value, GError **error)
+{
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (model), FALSE);
+	
+	return gda_data_model_iter_set_value_at (iter, col, value, error);
+}
+
+
 /**
  * gda_data_model_append_values:
  * @model: a #GdaDataModel object.
- * @values: (element-type GObject.Value) (allow-none): #GList of #GValue* representing the row to add.  The
+ * @values: (element-type GLib.Value) (allow-none): #GList of #GValue* representing the row to add.  The
  *          length must match model's column count.  These #GValue
  *	    are value-copied (the user is still responsible for freeing them).
  * @error: a place to store errors, or %NULL
@@ -997,7 +1121,7 @@ gda_data_model_remove_row (GdaDataModel *model, gint row, GError **error)
 /**
  * gda_data_model_get_row_from_values:
  * @model: a #GdaDataModel object.
- * @values: (element-type GObject.Value): a list of #GValue values (no %NULL is allowed)
+ * @values: (element-type GLib.Value): a list of #GValue values (no %NULL is allowed)
  * @cols_index: (array): an array of #gint containing the column number to match each value of @values
  *
  * Returns the first row where all the values in @values at the columns identified at
@@ -1097,9 +1221,11 @@ gda_data_model_send_hint (GdaDataModel *model, GdaDataModelHint hint, const GVal
  * data models where some rows may be missing because the LDAP search has reached a limit
  * imposed by the LDAP server.
  *
- * Returns: (transfer none) (element-type GLib.Error) (array zero-terminated=1): a pointer to a %NULL terminated array of #GError, or %NULL.
+ * Returns: (transfer none) (element-type GError) (array zero-terminated=1): a pointer to a %NULL terminated array of #GError, or %NULL.
  *
  * Since: 4.2.6
+ *
+ * Virtual: i_get_exceptions
  */
 GError **
 gda_data_model_get_exceptions (GdaDataModel *model)
