@@ -110,6 +110,8 @@ struct _GdaDataProxyClass
  *	  <listitem><para>if new rows have been added to the proxy</para></listitem>
  *	</itemizedlist>
  *    </para></listitem>
+ *    <listitem><para>Any #GdaDataModelIter iterator created will only make appear the colmuns as present in the proxied
+ *        data model, not any of the other columns</para></listitem>
  *  </itemizedlist>
  *  This situation is illustrated in the following schema, where there is a direct mapping between the proxy's
  *  rows and the proxied data model's rows:
@@ -182,6 +184,7 @@ struct _GdaDataProxyClass
 
 GType             gda_data_proxy_get_type                 (void) G_GNUC_CONST;
 GObject          *gda_data_proxy_new                      (GdaDataModel *model);
+GdaDataProxy     *gda_data_proxy_new_with_data_model      (GdaDataModel *model);
 
 GdaDataModel     *gda_data_proxy_get_proxied_model        (GdaDataProxy *proxy);
 gint              gda_data_proxy_get_proxied_model_n_cols (GdaDataProxy *proxy);
