@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2009 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2011 Murray Cumming <murrayc@murrayc.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,24 +20,6 @@
 
 #include "gda-types.h"
 
-GType
-_gda_error_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0)) {
-		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
-		g_static_mutex_lock (&registering);
-                if (type == 0)
-			type = g_boxed_type_register_static ("GdaError",
-							     (GBoxedCopyFunc) g_error_copy,
-							     (GBoxedFreeFunc) g_error_free);
-		g_static_mutex_unlock (&registering);
-	}
-
-	return type;
-}
-
 GType
 _gda_slist_get_type (void)
 {
@@ -53,18 +36,4 @@ _gda_slist_get_type (void)
 	return type;
 }
 
-GType
-_gda_meta_context_get_type (void)
-{
-	static GType type = 0;
 
-	if (G_UNLIKELY (type == 0)) {
-		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
-		g_static_mutex_lock (&registering);
-                if (type == 0)
-			type = g_pointer_type_register_static ("GdaMetaContext");
-		g_static_mutex_unlock (&registering);
-	}
-
-	return type;
-}
