@@ -469,7 +469,7 @@ _gda_sqlite_meta__udt (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *cnc
 	gint i, nrows;
 
 	/* get connection's private data */
-	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data_error (cnc, error);
 	if (!cdata)
 		return FALSE;
 
@@ -524,7 +524,7 @@ _gda_sqlite_meta_udt (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *cnc,
 	GHashTable *added_hash;
 
 	/* get connection's private data */
-	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data_error (cnc, error);
 	if (!cdata)
 		return FALSE;
 
@@ -721,18 +721,20 @@ _gda_sqlite_meta_schemata (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection
 		if (!schema_name_n || 
 		    !gda_value_compare (schema_name_n, cvalue)) {
 			const gchar *cstr;
-			GValue *v1;
+			GValue *v1, *v2;
 
 			cstr = g_value_get_string (cvalue); /* VMA */
 			if (!cstr || !strncmp (cstr, TMP_DATABASE_NAME, 4))
 				continue;
 
 			g_value_set_boolean ((v1 = gda_value_new (G_TYPE_BOOLEAN)), FALSE);
-			retval = append_a_row (model, error, 4, 
+			g_value_set_boolean ((v2 = gda_value_new (G_TYPE_BOOLEAN)), TRUE);
+			retval = append_a_row (model, error, 5, 
 					       FALSE, catalog_value,
 					       TRUE, new_caseless_value (cvalue), 
 					       FALSE, NULL,
-					       TRUE, v1);
+					       TRUE, v1,
+					       TRUE, v2);
 		}
 	}
 	g_object_unref (tmpmodel);
@@ -990,7 +992,7 @@ fill_columns_model (GdaConnection *cnc, SqliteConnectionData *cdata,
 		const gchar *this_col_name;
 		const GValue *this_col_pname;
 		GValue *nthis_col_pname;
-		GType gtype = G_TYPE_INVALID;
+		GType gtype = GDA_TYPE_NULL;
 		const GValue *cvalue;
 		
 		this_col_pname = gda_data_model_get_value_at (tmpmodel, 1, i, error);
@@ -1067,7 +1069,7 @@ fill_columns_model (GdaConnection *cnc, SqliteConnectionData *cdata,
 			gtype = pg ? (GType) *pg : GDA_TYPE_NULL;
 			g_free (tmp);
 		}
-		if (gtype == G_TYPE_INVALID)
+		if (gtype == GDA_TYPE_NULL)
 			/* default to string if nothing else */
 			g_value_set_string ((v6 = gda_value_new (G_TYPE_STRING)), "string");
 		else
@@ -1118,7 +1120,7 @@ _gda_sqlite_meta__columns (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection
 	gint i, nrows;
 	SqliteConnectionData *cdata;
 	
-	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data_error (cnc, error);
 	if (!cdata)
 		return FALSE;
 
@@ -1200,7 +1202,7 @@ _gda_sqlite_meta_columns (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection *
 	GdaDataModel *mod_model = NULL;
 	SqliteConnectionData *cdata;
 	
-	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data_error (cnc, error);
 	if (!cdata)
 		return FALSE;
 
@@ -1470,7 +1472,7 @@ _gda_sqlite_meta__constraints_tab (G_GNUC_UNUSED GdaServerProvider *prov, GdaCon
 	gint i, nrows;
 	SqliteConnectionData *cdata;
 	
-	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data_error (cnc, error);
 	if (!cdata)
 		return FALSE;
 
@@ -1552,7 +1554,7 @@ _gda_sqlite_meta_constraints_tab (G_GNUC_UNUSED GdaServerProvider *prov, GdaConn
 	GdaDataModel *mod_model = NULL;
 	SqliteConnectionData *cdata;
 	
-	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data_error (cnc, error);
 	if (!cdata)
 		return FALSE;
 
@@ -1662,7 +1664,7 @@ _gda_sqlite_meta__constraints_ref (G_GNUC_UNUSED GdaServerProvider *prov, GdaCon
 	SqliteConnectionData *cdata;
 	gint fk_enforced = -1;
 	
-	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data_error (cnc, error);
 	if (!cdata)
 		return FALSE;
 
@@ -1762,7 +1764,7 @@ _gda_sqlite_meta_constraints_ref (G_GNUC_UNUSED GdaServerProvider *prov, GdaConn
 	GdaDataModel *mod_model = NULL;
 	SqliteConnectionData *cdata;
 	
-	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data_error (cnc, error);
 	if (!cdata)
 		return FALSE;
 
@@ -2007,7 +2009,7 @@ _gda_sqlite_meta__key_columns (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnect
 	gint i, nrows;
 	SqliteConnectionData *cdata;
 	
-	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data_error (cnc, error);
 	if (!cdata)
 		return FALSE;
 
@@ -2123,7 +2125,7 @@ _gda_sqlite_meta_key_columns (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnecti
 	GdaDataModel *mod_model = NULL;
 	SqliteConnectionData *cdata;
 	
-	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data_error (cnc, error);
 	if (!cdata)
 		return FALSE;
 
@@ -2244,7 +2246,7 @@ _gda_sqlite_meta_routines (G_GNUC_UNUSED GdaServerProvider *prov, GdaConnection
 	GdaDataModel *tmpmodel, *mod_model;
 	gint i, nrows;
 
-	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data (cnc);
+	cdata = (SqliteConnectionData*) gda_connection_internal_get_provider_data_error (cnc, error);
 	if (!cdata)
 		return FALSE;
 
