@@ -293,8 +293,7 @@ jni_wrapper_instantiate_object (JNIEnv *jenv, jclass klass, const gchar *signatu
 	va_end (args);
 	
 	if (jni_wrapper_handle_exception (jenv, NULL, NULL, error)) {
-		g_value_unset (retval);
-		g_free (retval);
+		gda_value_free (retval);
 		retval = NULL;
 	}
 
@@ -355,8 +354,7 @@ jni_wrapper_handle_exception (JNIEnv *jenv, gint *out_error_code, gchar **out_sq
 				if (res) {
 					if (G_VALUE_TYPE (res) == G_TYPE_INT)
 						*out_error_code = g_value_get_int (res);
-					g_value_unset (res);
-					g_free (res);
+					gda_value_free (res);
 				}
 			}
 			if (out_sql_state) {
@@ -367,8 +365,7 @@ jni_wrapper_handle_exception (JNIEnv *jenv, gint *out_error_code, gchar **out_sq
 				if (res) {
 					if (G_VALUE_TYPE (res) == G_TYPE_STRING)
 						*out_sql_state = g_value_dup_string (res);
-					g_value_unset (res);
-					g_free (res);
+					gda_value_free (res);
 				}
 			}
 		}
@@ -389,9 +386,7 @@ jni_wrapper_handle_exception (JNIEnv *jenv, gint *out_error_code, gchar **out_sq
 					gda_value_free (res);
 				}
 				else {
-					if (G_VALUE_TYPE (res) != 0) /* GDA_TYPE_NULL */
-						g_value_unset (res);
-					g_free (res);
+					gda_value_free (res);
 					goto fallback;
 				}
 			}
@@ -402,10 +397,7 @@ jni_wrapper_handle_exception (JNIEnv *jenv, gint *out_error_code, gchar **out_sq
 			goto fallback;
 	}
 
-	if (exc_value) {
-		g_value_unset (exc_value);
-		g_free (exc_value);
-	}
+	gda_value_free (exc_value);
 
 	return TRUE;
 
@@ -504,6 +496,7 @@ jni_wrapper_method_call (JNIEnv *jenv, JniWrapperMethod *method, GValue *object,
 			(*jenv)->CallStaticVoidMethodV (jenv, method->klass, method->mid, args);
 		else
 			(*jenv)->CallVoidMethodV (jenv, jobj, method->mid, args);
+		gda_value_set_null (retval);
 		break;
 	case '[':
 	case 'L': 
@@ -563,12 +556,21 @@ jni_wrapper_method_call (JNIEnv *jenv, JniWrapperMethod *method, GValue *object,
 		break;
 	case 'B':
 		g_value_init (retval, G_TYPE_CHAR);
+#if GLIB_CHECK_VERSION(2,31,7)
+		if (method->is_static)
+			g_value_set_schar (retval,
+					   (*jenv)->CallStaticByteMethodV (jenv, method->klass, method->mid, args));
+		else
+			g_value_set_schar (retval,
+					   (*jenv)->CallByteMethodV (jenv, jobj, method->mid, args));
+#else
 		if (method->is_static)
 			g_value_set_char (retval,
 					  (*jenv)->CallStaticByteMethodV (jenv, method->klass, method->mid, args));
 		else
 			g_value_set_char (retval,
 					  (*jenv)->CallByteMethodV (jenv, jobj, method->mid, args));
+#endif
 		break;
 	case 'C':
 		g_value_init (retval, G_TYPE_INT); // FIXME: should be an unsigned 16 bits value
@@ -630,9 +632,7 @@ jni_wrapper_method_call (JNIEnv *jenv, JniWrapperMethod *method, GValue *object,
 	va_end (args);
 
 	if (jni_wrapper_handle_exception (jenv, out_error_code, out_sql_state, error)) {
-		if (G_VALUE_TYPE (retval) != 0)
-			g_value_unset (retval);
-		g_free (retval);
+		gda_value_free (retval);
 		return NULL;
 	}
  
@@ -777,12 +777,21 @@ jni_wrapper_field_get (JNIEnv *jenv, JniWrapperField *field, GValue *object, GEr
 		break;
 	case 'B':
 		g_value_init (retval, G_TYPE_CHAR);
+#if GLIB_CHECK_VERSION(2,31,7)
+		if (field->is_static)
+			g_value_set_schar (retval,
+					   (*jenv)->GetStaticByteField (jenv, field->klass, field->fid));
+		else
+			g_value_set_schar (retval,
+					   (*jenv)->GetByteField (jenv, jobj, field->fid));
+#else
 		if (field->is_static)
 			g_value_set_char (retval,
 					  (*jenv)->GetStaticByteField (jenv, field->klass, field->fid));
 		else
 			g_value_set_char (retval,
 					  (*jenv)->GetByteField (jenv, jobj, field->fid));
+#endif
 		break;
 	case 'C':
 		g_value_init (retval, G_TYPE_INT); // FIXME: should be an unsigned 16 bits value
@@ -843,9 +852,7 @@ jni_wrapper_field_get (JNIEnv *jenv, JniWrapperField *field, GValue *object, GEr
 	}
 
 	if (jni_wrapper_handle_exception (jenv, NULL, NULL, error)) {
-		if (G_VALUE_TYPE (retval) != 0)
-			g_value_unset (retval);
-		g_free (retval);
+		gda_value_free (retval);
 		return NULL;
 	}
  
@@ -934,10 +941,17 @@ jni_wrapper_field_set (JNIEnv *jenv, JniWrapperField *field,
 	case 'B':
 		if (G_VALUE_TYPE (value) != G_TYPE_CHAR)
 			goto wrong_type;
+#if GLIB_CHECK_VERSION(2,31,7)
+		if (field->is_static)
+			(*jenv)->SetStaticByteField (jenv, field->klass, field->fid, g_value_get_schar (value));
+		else
+			(*jenv)->SetByteField (jenv, jobj, field->fid, g_value_get_schar (value));
+#else
 		if (field->is_static)
 			(*jenv)->SetStaticByteField (jenv, field->klass, field->fid, g_value_get_char (value));
 		else
 			(*jenv)->SetByteField (jenv, jobj, field->fid, g_value_get_char (value));
+#endif
 		break;
 	case 'C':
 		if (G_VALUE_TYPE (value) != G_TYPE_INT)
