@@ -362,6 +362,8 @@ gda_ldap_provider_open_connection (GdaServerProvider *provider, GdaConnection *c
 	const gchar *user = NULL;
 	gchar *dnuser = NULL;
         const gchar *pwd = NULL;
+        const gchar *time_limit = NULL;
+        const gchar *size_limit = NULL;
         const gchar *tls_method = NULL;
         const gchar *tls_cacert = NULL;
 	int rtls_method = -1;
@@ -416,6 +418,8 @@ gda_ldap_provider_open_connection (GdaServerProvider *provider, GdaConnection *c
 			return FALSE;
 		}
 	}
+	time_limit = gda_quark_list_find (params, "TIME_LIMIT");
+	size_limit = gda_quark_list_find (params, "SIZE_LIMIT");
 
 	/* open LDAP connection */
 	LdapConnectionData *cdata;
@@ -456,7 +460,7 @@ gda_ldap_provider_open_connection (GdaServerProvider *provider, GdaConnection *c
 	else
 		url = g_strdup_printf ("ldap://%s:%d", host, rport);
 
-	if (! gda_ldap_parse_dn (user, NULL) && *user) {
+	if (user && *user && ! gda_ldap_parse_dn (user, NULL)) {
 		/* analysing the @user parameter */
 		/* the user name is not a DN => we need to fetch the DN of the entry
 		 * using filters defined in the "mappings" array @user */
@@ -497,8 +501,11 @@ gda_ldap_provider_open_connection (GdaServerProvider *provider, GdaConnection *c
         }
 
 	cdata = g_new0 (LdapConnectionData, 1);
+	cdata->keep_bound_count = 0;
 	cdata->handle = ld;
 	cdata->url = url;
+	cdata->time_limit = 0;
+	cdata->size_limit = 0;
 	cdata->base_dn = g_strdup (base_dn);
 	if (use_cache)
 		cdata->attributes_cache_file = compute_data_file_name (params, TRUE, "attrs");
@@ -518,7 +525,32 @@ gda_ldap_provider_open_connection (GdaServerProvider *provider, GdaConnection *c
 			return FALSE;
 		}
         }
-	res = ldap_set_option (cdata->handle, LDAP_OPT_RESTART, LDAP_OPT_ON);
+
+	/* time limit */
+	if (time_limit && *time_limit) {
+		int limit = atoi (time_limit);
+		res = ldap_set_option (cdata->handle, LDAP_OPT_TIMELIMIT, &limit);
+		if (res != LDAP_SUCCESS) {
+			gda_connection_add_event_string (cnc, ldap_err2string (res));
+			gda_ldap_free_cnc_data (cdata);
+			g_free (dnuser);
+			return FALSE;
+		}
+		cdata->time_limit = limit;
+	}
+
+	/* size limit */
+	if (size_limit && *size_limit) {
+		int limit = atoi (size_limit);
+		res = ldap_set_option (cdata->handle, LDAP_OPT_SIZELIMIT, &limit);
+		if (res != LDAP_SUCCESS) {
+			gda_connection_add_event_string (cnc, ldap_err2string (res));
+			gda_ldap_free_cnc_data (cdata);
+			g_free (dnuser);
+			return FALSE;
+		}
+		cdata->size_limit = limit;
+	}
 
 	/* authentication */
 	struct berval cred;
@@ -532,14 +564,31 @@ gda_ldap_provider_open_connection (GdaServerProvider *provider, GdaConnection *c
 		g_free (dnuser);
                 return FALSE;
 	}
-	if (pwd)
-		cdata->pass = g_strdup (pwd);
+	if (pwd) {
+		gchar *tmp;
+		tmp = g_strdup_printf ("PASSWORD=%s", pwd);
+		cdata->auth = gda_quark_list_new_from_string (tmp);
+		g_free (tmp);
+	}
 	if (dnuser) {
-		cdata->user = dnuser;
+		gchar *tmp;
+		tmp = g_strdup_printf ("USERNAME=%s", dnuser);
+		if (cdata->auth)
+			gda_quark_list_add_from_string (cdata->auth, tmp, FALSE);
+		else
+			cdata->auth = gda_quark_list_new_from_string (tmp);
+		g_free (tmp);
 		dnuser = NULL;
 	}
-	else if (user)
-		cdata->user = g_strdup (user);
+	else if (user) {
+		gchar *tmp;
+		tmp = g_strdup_printf ("USERNAME=%s", user);
+		if (cdata->auth)
+			gda_quark_list_add_from_string (cdata->auth, tmp, FALSE);
+		else
+			cdata->auth = gda_quark_list_new_from_string (tmp);
+		g_free (tmp);
+	}
 
 	/* set startup file name */
 	gchar *fname;
@@ -559,16 +608,46 @@ gda_ldap_provider_open_connection (GdaServerProvider *provider, GdaConnection *c
                 return FALSE;
         }
 
+	gda_ldap_may_unbind (cdata);
 	return TRUE;
 }
 
+/*
+ * Unbinds the connection if possible (i.e. if cdata->keep_bound_count is 0)
+ * This allows to avoid keeping the connection to the LDAP server if unused
+ */
+void
+gda_ldap_may_unbind (LdapConnectionData *cdata)
+{
+	if (!cdata || (cdata->keep_bound_count > 0))
+		return;
+	if (cdata->handle) {
+                ldap_unbind_ext (cdata->handle, NULL, NULL);
+		cdata->handle = NULL;
+	}
+}
+
+/*
+ * Makes sure the connection is opened
+ */
+gboolean
+gda_ldap_ensure_bound (LdapConnectionData *cdata, GError **error)
+{
+	if (!cdata)
+		return FALSE;
+	else if (cdata->handle)
+		return TRUE;
+
+	return gda_ldap_rebind (cdata, error);
+}
+
 /*
  * Reopens a connection after the server has closed it (possibly because of a timeout)
  *
  * If it fails, then @cdata is left unchanged, otherwise it is modified to be useable again.
  */
 gboolean
-gda_ldap_silently_rebind (LdapConnectionData *cdata)
+gda_ldap_rebind (LdapConnectionData *cdata, GError **error)
 {
 	if (!cdata)
 		return FALSE;
@@ -577,8 +656,11 @@ gda_ldap_silently_rebind (LdapConnectionData *cdata)
 	LDAP *ld;
 	int res;
 	res = ldap_initialize (&ld, cdata->url);
-	if (res != LDAP_SUCCESS)
+	if (res != LDAP_SUCCESS) {
+		g_set_error (error, GDA_CONNECTION_ERROR, GDA_CONNECTION_OPEN_ERROR,
+			     "%s", ldap_err2string (res));
 		return FALSE;
+	}
 
 	/* set protocol version to 3 by default */
 	int version = LDAP_VERSION3;
@@ -589,6 +671,8 @@ gda_ldap_silently_rebind (LdapConnectionData *cdata)
 			res = ldap_set_option (ld, LDAP_OPT_PROTOCOL_VERSION, &version);
 		}
 		if (res != LDAP_SUCCESS) {
+			g_set_error (error, GDA_CONNECTION_ERROR, GDA_CONNECTION_OPEN_ERROR,
+				     "%s", ldap_err2string (res));
 			ldap_unbind_ext (ld, NULL, NULL);
 			return FALSE;
 		}
@@ -596,13 +680,43 @@ gda_ldap_silently_rebind (LdapConnectionData *cdata)
 
 	/* authentication */
 	struct berval cred;
-	const gchar *pwd;
-	pwd = cdata->pass;
+	const gchar *pwd = "";
+	const gchar *user = "";
+	if (cdata->auth)
+		pwd = gda_quark_list_find (cdata->auth, "PASSWORD");
         memset (&cred, 0, sizeof (cred));
         cred.bv_len = pwd && *pwd ? strlen (pwd) : 0;
         cred.bv_val = pwd && *pwd ? (char *) pwd : NULL;
-	res = ldap_sasl_bind_s (ld, cdata->user, NULL, &cred, NULL, NULL, NULL);
+
+	if (cdata->auth)
+		user = gda_quark_list_find (cdata->auth, "USERNAME");
+	res = ldap_sasl_bind_s (ld, user, NULL, &cred, NULL, NULL, NULL);
+	if (cdata->auth)
+		gda_quark_list_protect_values (cdata->auth);
+
 	if (res != LDAP_SUCCESS) {
+		g_set_error (error, GDA_CONNECTION_ERROR, GDA_CONNECTION_OPEN_ERROR,
+			     "%s", ldap_err2string (res));
+		ldap_unbind_ext (ld, NULL, NULL);
+                return FALSE;
+	}
+
+	/* time limit */
+	int limit = cdata->time_limit;
+	res = ldap_set_option (cdata->handle, LDAP_OPT_TIMELIMIT, &limit);
+	if (res != LDAP_SUCCESS) {
+		g_set_error (error, GDA_CONNECTION_ERROR, GDA_CONNECTION_OPEN_ERROR,
+			     "%s", ldap_err2string (res));
+		ldap_unbind_ext (ld, NULL, NULL);
+                return FALSE;
+	}
+
+	/* size limit */
+	limit = cdata->size_limit;
+	res = ldap_set_option (cdata->handle, LDAP_OPT_SIZELIMIT, &limit);
+	if (res != LDAP_SUCCESS) {
+		g_set_error (error, GDA_CONNECTION_ERROR, GDA_CONNECTION_OPEN_ERROR,
+			     "%s", ldap_err2string (res));
 		ldap_unbind_ext (ld, NULL, NULL);
                 return FALSE;
 	}
@@ -653,8 +767,7 @@ gda_ldap_provider_get_database (GdaServerProvider *provider, GdaConnection *cnc)
         cdata = (LdapConnectionData*) gda_virtual_connection_internal_get_provider_data (GDA_VIRTUAL_CONNECTION (cnc));
         if (!cdata)
                 return NULL;
-        TO_IMPLEMENT;
-        return NULL;
+        return cdata->base_dn;
 }
 
 /*
@@ -851,8 +964,8 @@ gda_ldap_free_cnc_data (LdapConnectionData *cdata)
 	g_free (cdata->base_dn);
 	g_free (cdata->server_version);
 	g_free (cdata->url);
-	g_free (cdata->user);
-	g_free (cdata->pass);
+	if (cdata->auth)
+		gda_quark_list_free (cdata->auth);
 	g_free (cdata);
 }
 
