@@ -53,7 +53,7 @@ prov_test_common_setup ()
 			if (error->domain != 0) {
 				gchar *str = g_strdup_printf ("Could not setup connection: %s", 
 							      error->message ? error->message : "No detail");
-				fail (str);
+				g_warning ("%s", str);
 				g_free (str);
 				number_failed++;
 			}
@@ -68,7 +68,7 @@ prov_test_common_setup ()
 		if (!test_cnc_setup_db_structure (cnc, file, &error)) {
 			gchar *str = g_strdup_printf ("Could not setup database structure: %s", 
 						      error && error->message ? error->message : "No detail");
-			fail (str);
+			g_warning ("%s", str);
 			g_free (str);
 			if (error)
 				g_error_free (error);
@@ -188,7 +188,35 @@ prov_test_common_check_meta ()
 		number_failed++;
 		goto theend;
 	}
-
+	/* Update meta store by context */
+	GdaMetaContext *ctx;
+	ctx = gda_meta_context_new ();
+	gchar *meta_tables[] = {"_tables", "_attributes", "_information_schema_catalog_name",
+						"_schemata", "_builtin_data_types", "_udt", "_udt_columns",
+						"_enums", "_element_types", "_domains", "_views", "_collations",
+						"_character_sets", "_routines", "_triggers", "_columns",
+						"_table_constraints", "_referential_constraints",
+						"_key_column_usage", "__declared_fk", "_check_column_usage",
+						"_view_column_usage", "_domain_constraints", "_parameters",
+						"_routine_columns", "_table_indexes", "_index_column_usage",
+						""};
+	for (i = 0; g_strcmp0 (meta_tables[i],"") != 0; i++) {
+		gda_meta_context_set_table (ctx, meta_tables[i]);
+#ifdef CHECK_EXTRA_INFO
+		g_print ("Updating the meta store for table '%s'\n", meta_tables[i]);
+#endif
+		if (! gda_connection_update_meta_store (cnc, ctx, &gerror)) {
+#ifdef CHECK_EXTRA_INFO
+			g_warning ("Can't update meta store ( on table %s): %s\n",
+				   meta_tables[i], gerror && gerror->message ? gerror->message : "???");
+#endif
+			g_error_free (gerror);
+			number_failed++;
+			goto theend;
+		}
+	}
+	gda_meta_context_free (ctx);
+	
 	for (i = 0, list = tables; list; i++, list = list->next) {
 		GdaDataModel *model;
 		gchar *tmp;
@@ -280,7 +308,7 @@ prov_test_common_check_meta_identifiers (gboolean case_sensitive, gboolean updat
         operation = gda_server_provider_create_operation (provider, cnc,
 							  GDA_SERVER_OPERATION_DROP_TABLE, NULL, &error);
         g_assert (operation);
-        gda_server_operation_set_value_at (operation, table_name, NULL, "/TABLE_DESC_P/TABLE_NAME");
+        gda_server_operation_set_value_at_path (operation, table_name, "/TABLE_DESC_P/TABLE_NAME", NULL);
         gda_server_provider_perform_operation (provider, cnc, operation, NULL);
 	g_object_unref (operation);
 
@@ -396,9 +424,9 @@ prov_test_common_check_meta_identifiers (gboolean case_sensitive, gboolean updat
 	operation = gda_server_provider_create_operation (provider, cnc,
 							  GDA_SERVER_OPERATION_ADD_COLUMN, NULL, &error);
         g_assert (operation);
-        gda_server_operation_set_value_at (operation, table_name, NULL, "/COLUMN_DEF_P/TABLE_NAME");
-        gda_server_operation_set_value_at (operation, field_name, NULL, "/COLUMN_DEF_P/COLUMN_NAME");
-        gda_server_operation_set_value_at (operation, "int", NULL, "/COLUMN_DEF_P/COLUMN_TYPE");
+        gda_server_operation_set_value_at_path (operation, table_name, "/COLUMN_DEF_P/TABLE_NAME", NULL);
+        gda_server_operation_set_value_at (operation, field_name, "/COLUMN_DEF_P/COLUMN_NAME", NULL);
+        gda_server_operation_set_value_at (operation, "int", "/COLUMN_DEF_P/COLUMN_TYPE", NULL);
         if (! gda_server_provider_perform_operation (provider, cnc, operation, &error)) {
 #ifdef CHECK_EXTRA_INFO
                 g_warning ("perform_operation(ADD_COLUMN) failed: %s\n", error && error->message ? 
@@ -612,6 +640,7 @@ prov_test_common_check_data_select ()
 		columns = g_slist_append (columns, gda_data_model_describe_column (model, i));
 
 	/* change param */
+	g_object_set (model, "auto-reset", TRUE, NULL);
 	if (! gda_set_set_holder_value (params, &error, "theid", 9)) {
                 number_failed++;
                 goto out;
@@ -624,7 +653,7 @@ prov_test_common_check_data_select ()
 		goto out;
 	}
 
-	/* chech the columns haven't changed */
+	/* check the columns haven't changed */
 	ncols = gda_data_model_get_n_columns (model);
 	if (i != ncols) {
 		g_set_error (&error, 0, 0,
