@@ -92,6 +92,47 @@ wrapper_job_free (WrapperJob *wj)
 	g_free (wj);
 }
 
+#ifdef GDA_DEBUG_MUTEX
+static void
+my_lock (GMutex *mutex, gint where)
+{
+	GTimer *timer;
+	g_print ("wait to lock %p (th %p)@line %d\n", mutex, g_thread_self(), where);
+	timer = g_timer_new ();
+	g_mutex_lock (mutex);
+	g_timer_stop (timer);
+
+	if (g_timer_elapsed (timer, NULL) > 2.0)
+		g_print ("WARN: locking %p (th %p)@line %d: %02f\n", mutex, g_thread_self(), where,
+			 g_timer_elapsed (timer, NULL));
+	g_print ("tmp LOC %p (th %p)@line %d took %02f\n", mutex, g_thread_self(), where,
+		 g_timer_elapsed (timer, NULL));
+	g_timer_destroy (timer);
+}
+static void
+my_unlock (GMutex *mutex, gint where)
+{
+	g_mutex_unlock (mutex);
+	g_print ("tmp UNL %p (th %p)@line %d\n", mutex, g_thread_self(), where);
+}
+
+#if GLIB_CHECK_VERSION(2,31,7)
+#define MUTEX_LOCK(bcnc) g_mutex_lock (&((bcnc)->priv->mstruct_mutex))
+#define MUTEX_UNLOCK(bcnc) g_mutex_unlock (&((bcnc)->priv->mstruct_mutex))
+#else
+#define MUTEX_LOCK(bcnc) my_lock ((bcnc)->priv->p_mstruct_mutex,__LINE__)
+#define MUTEX_UNLOCK(bcnc) my_unlock ((bcnc)->priv->p_mstruct_mutex,__LINE__)
+#endif
+#else /* GDA_DEBUG_MUTEX */
+#if GLIB_CHECK_VERSION(2,31,7)
+#define MUTEX_LOCK(bcnc) g_mutex_lock (&((bcnc)->priv->mstruct_mutex))
+#define MUTEX_UNLOCK(bcnc) g_mutex_unlock (&((bcnc)->priv->mstruct_mutex))
+#else
+#define MUTEX_LOCK(bcnc) g_mutex_lock ((bcnc)->priv->p_mstruct_mutex)
+#define MUTEX_UNLOCK(bcnc) g_mutex_unlock ((bcnc)->priv->p_mstruct_mutex)
+#endif
+#endif /* GDA_DEBUG_MUTEX */
+
 /*
  * Returns: %TRUE if current timer should be removed
  */
@@ -353,7 +394,11 @@ browser_connection_init (BrowserConnection *bcnc)
 	bcnc->priv->parser = NULL;
 	bcnc->priv->variables = NULL;
 	memset (&(bcnc->priv->dsn_info), 0, sizeof (GdaDsnInfo));
+#if GLIB_CHECK_VERSION(2,31,7)
+	g_mutex_init (&bcnc->priv->mstruct_mutex);
+#else
 	bcnc->priv->p_mstruct_mutex = g_mutex_new ();
+#endif
 	bcnc->priv->p_mstruct_list = NULL;
 	bcnc->priv->c_mstruct = NULL;
 	bcnc->priv->mstruct = NULL;
@@ -395,7 +440,7 @@ wrapper_meta_struct_sync (BrowserConnection *bcnc, GError **error)
 	gboolean retval = TRUE;
 	GdaMetaStruct *mstruct;
 
-	g_mutex_lock (bcnc->priv->p_mstruct_mutex);
+	MUTEX_LOCK (bcnc);
 	g_assert (bcnc->priv->p_mstruct_list);
 	mstruct = (GdaMetaStruct *) bcnc->priv->p_mstruct_list->data;
 	/*g_print ("%s() for GdaMetaStruct %p\n", __FUNCTION__, mstruct);*/
@@ -404,7 +449,7 @@ wrapper_meta_struct_sync (BrowserConnection *bcnc, GError **error)
 	if (bcnc->priv->p_mstruct_list) {
 		/* don't care about this one */
 		g_object_unref (G_OBJECT (mstruct));
-		g_mutex_unlock (bcnc->priv->p_mstruct_mutex);
+		MUTEX_UNLOCK (bcnc);
 		return GINT_TO_POINTER (3);
 	}
 	else {
@@ -414,7 +459,7 @@ wrapper_meta_struct_sync (BrowserConnection *bcnc, GError **error)
 
 		/*g_print ("Meta struct sync for %p\n", mstruct);*/
 		retval = gda_meta_struct_complement_all (mstruct, error);
-		g_mutex_unlock (bcnc->priv->p_mstruct_mutex);
+		MUTEX_UNLOCK (bcnc);
 	}
 
 #ifdef GDA_DEBUG_NO
@@ -445,13 +490,13 @@ meta_changed_cb (G_GNUC_UNUSED GdaThreadWrapper *wrapper,
 	guint job_id;
 	GError *lerror = NULL;
 	GdaMetaStruct *mstruct;
-	
-	g_mutex_lock (bcnc->priv->p_mstruct_mutex);
+
+	MUTEX_LOCK (bcnc);
 	mstruct = gda_meta_struct_new (gda_connection_get_meta_store (bcnc->priv->cnc),
 				       GDA_META_STRUCT_FEATURE_ALL);
 	bcnc->priv->p_mstruct_list = g_slist_append (bcnc->priv->p_mstruct_list, mstruct);
 	/*g_print ("%s() Added %p to p_mstruct_list\n", __FUNCTION__, mstruct);*/
-	g_mutex_unlock (bcnc->priv->p_mstruct_mutex);
+	MUTEX_UNLOCK (bcnc);
 	job_id = gda_thread_wrapper_execute (bcnc->priv->wrapper,
 					     (GdaThreadWrapperFunc) wrapper_meta_struct_sync,
 					     g_object_ref (bcnc), g_object_unref, &lerror);
@@ -479,6 +524,108 @@ browser_connection_meta_data_changed (BrowserConnection *bcnc)
 	meta_changed_cb (NULL, NULL, NULL, 0, NULL, NULL, bcnc);
 }
 
+static gpointer
+wrapper_have_meta_store_ready (BrowserConnection *bcnc, GError **error)
+{
+	gchar *dict_file_name = NULL;
+	gboolean update_store = FALSE;
+	GdaMetaStore *store;
+	gchar *cnc_string, *cnc_info;
+
+	g_object_get (G_OBJECT (bcnc->priv->cnc),
+		      "dsn", &cnc_info,
+		      "cnc-string", &cnc_string, NULL);
+	dict_file_name = config_info_compute_dict_file_name (cnc_info ? gda_config_get_dsn_info (cnc_info) : NULL,
+							     cnc_string);
+	g_free (cnc_string);
+	if (dict_file_name) {
+		if (BROWSER_IS_VIRTUAL_CONNECTION (bcnc))
+			/* force meta store update in case of virtual connection */
+			update_store = TRUE;
+		else if (! g_file_test (dict_file_name, G_FILE_TEST_EXISTS))
+			update_store = TRUE;
+		store = gda_meta_store_new_with_file (dict_file_name);
+	}
+	else {
+		store = gda_meta_store_new (NULL);
+		if (store)
+			update_store = TRUE;
+	}
+	config_info_update_meta_store_properties (store, bcnc->priv->cnc);
+
+	bcnc->priv->dict_file_name = dict_file_name;
+	g_object_set (G_OBJECT (bcnc->priv->cnc), "meta-store", store, NULL);
+	if (update_store) {
+		gboolean retval;
+		GdaMetaContext context = {"_tables", 0, NULL, NULL};
+		retval = gda_connection_update_meta_store (bcnc->priv->cnc, &context, error);
+		if (!retval) {
+			g_object_unref (store);
+			return NULL;
+		}
+	}
+
+	gboolean retval = TRUE;
+	GdaMetaStruct *mstruct;
+	mstruct = gda_meta_struct_new (store, GDA_META_STRUCT_FEATURE_ALL);
+	MUTEX_LOCK (bcnc);
+	if (bcnc->priv->c_mstruct) {
+		g_object_unref (bcnc->priv->c_mstruct);
+		bcnc->priv->c_mstruct = NULL;
+	}
+	bcnc->priv->mstruct = mstruct;
+	retval = gda_meta_struct_complement_all (mstruct, error);
+	MUTEX_UNLOCK (bcnc);
+
+#ifdef GDA_DEBUG_NO
+	GSList *all, *list;
+	g_print ("%s() %p:\n", __FUNCTION__, bcnc->priv->mstruct);
+	all = gda_meta_struct_get_all_db_objects (bcnc->priv->mstruct);
+	for (list = all; list; list = list->next) {
+		GdaMetaDbObject *dbo = (GdaMetaDbObject *) list->data;
+		g_print ("DBO, Type %d: short=>[%s] schema=>[%s] full=>[%s]\n", dbo->obj_type,
+			 dbo->obj_short_name, dbo->obj_schema, dbo->obj_full_name);
+	}
+	g_slist_free (all);
+#endif
+	bcnc->priv->meta_store_signal =
+		gda_thread_wrapper_connect_raw (bcnc->priv->wrapper, store, "meta-changed",
+						FALSE, FALSE,
+						(GdaThreadWrapperCallback) meta_changed_cb,
+						bcnc);
+	g_object_unref (store);
+	return retval ? (void*) 0x01 : NULL;
+}
+
+typedef struct {
+        guint jid;
+        GMainLoop *loop;
+        GError **error;
+        GdaThreadWrapper *wrapper;
+
+        /* out */
+	gboolean retval;
+} MainloopData;
+
+static gboolean
+check_for_meta_store_ready (MainloopData *data)
+{
+	gpointer retval;
+        GError *lerror = NULL;
+
+        retval = gda_thread_wrapper_fetch_result (data->wrapper, FALSE, data->jid, &lerror);
+        if (retval || (!retval && lerror)) {
+                /* waiting is finished! */
+                data->retval = retval ? TRUE : FALSE;
+                if (lerror)
+                        g_propagate_error (data->error, lerror);
+                g_main_loop_quit (data->loop);
+                return FALSE;
+        }
+        return TRUE;
+}
+
+
 static void
 browser_connection_set_property (GObject *object,
 				 guint param_id,
@@ -508,80 +655,37 @@ browser_connection_set_property (GObject *object,
 								bcnc);
 
 
-			/* meta store */
-			gchar *dict_file_name = NULL;
-			gboolean update_store = FALSE;
-			GdaMetaStore *store;
-			gchar *cnc_string, *cnc_info;
-			
-			g_object_get (G_OBJECT (bcnc->priv->cnc),
-				      "dsn", &cnc_info,
-				      "cnc-string", &cnc_string, NULL);
-			dict_file_name = config_info_compute_dict_file_name (cnc_info ? gda_config_get_dsn_info (cnc_info) : NULL,
-									     cnc_string);
-			g_free (cnc_string);
-			if (dict_file_name) {
-				if (BROWSER_IS_VIRTUAL_CONNECTION (bcnc))
-					/* force meta store update in case of virtual connection */
-					update_store = TRUE;
-				else if (! g_file_test (dict_file_name, G_FILE_TEST_EXISTS))
-					update_store = TRUE;
-				store = gda_meta_store_new_with_file (dict_file_name);
+			/* meta store, open it in a sub thread to avoid locking the GTK thread */
+			gpointer retval;
+			GError *lerror = NULL;
+			guint jid;
+			jid = gda_thread_wrapper_execute (bcnc->priv->wrapper,
+							  (GdaThreadWrapperFunc) wrapper_have_meta_store_ready,
+							  (gpointer) bcnc,
+							  NULL, &lerror);
+			if (jid == 0) {
+				browser_show_error (NULL, _("Error while fetching meta data from the connection: %s"),
+						    lerror->message ? lerror->message : _("No detail"));
+				g_clear_error (&lerror);
 			}
 			else {
-				store = gda_meta_store_new (NULL);
-				if (store)
-					update_store = TRUE;
-			}
-			config_info_update_meta_store_properties (store, bcnc->priv->cnc);
+				GMainLoop *loop;
+				MainloopData data;
 
-			bcnc->priv->dict_file_name = dict_file_name;
-			g_object_set (G_OBJECT (bcnc->priv->cnc), "meta-store", store, NULL);
-			if (update_store) {
-				GError *lerror = NULL;
-				guint job_id;
-				job_id = gda_thread_wrapper_execute (bcnc->priv->wrapper,
-								     (GdaThreadWrapperFunc) wrapper_meta_store_update,
-								     g_object_ref (bcnc), g_object_unref, &lerror);
-				if (job_id > 0)
-					push_wrapper_job (bcnc, job_id, JOB_TYPE_META_STORE_UPDATE,
-							  _("Getting database schema information"),
-							  NULL, NULL);
-				else if (lerror) {
+				loop = g_main_loop_new (NULL, FALSE);
+				data.jid = jid;
+				data.loop = loop;
+				data.error = &lerror;
+				data.wrapper = bcnc->priv->wrapper;
+				g_timeout_add (200, (GSourceFunc) check_for_meta_store_ready, &data);
+				g_main_loop_run (loop);
+				g_main_loop_unref (loop);
+				if (!data.retval) {
 					browser_show_error (NULL, _("Error while fetching meta data from the connection: %s"),
 							    lerror->message ? lerror->message : _("No detail"));
-					g_error_free (lerror);
+					g_clear_error (&lerror);
 				}
 			}
-			else {
-				guint job_id;
-				GError *lerror = NULL;
-				GdaMetaStruct *mstruct;
-
-				mstruct = gda_meta_struct_new (store, GDA_META_STRUCT_FEATURE_ALL);
-				bcnc->priv->p_mstruct_list = g_slist_append (bcnc->priv->p_mstruct_list,
-									     mstruct);
-				/*g_print ("%s() Added %p to p_mstruct_list\n", __FUNCTION__, mstruct);*/
-				job_id = gda_thread_wrapper_execute (bcnc->priv->wrapper,
-								     (GdaThreadWrapperFunc) wrapper_meta_struct_sync,
-								     g_object_ref (bcnc), g_object_unref, &lerror);
-				if (job_id > 0)
-					push_wrapper_job (bcnc, job_id, JOB_TYPE_META_STRUCT_SYNC,
-							  _("Analysing database schema"),
-							  NULL, NULL);
-				else if (lerror) {
-					browser_show_error (NULL, _("Error while fetching meta data from the connection: %s"),
-							    lerror->message ? lerror->message : _("No detail"));
-					g_error_free (lerror);
-				}
-				g_object_unref (store);
-			}
-			bcnc->priv->meta_store_signal =
-				gda_thread_wrapper_connect_raw (bcnc->priv->wrapper, store, "meta-changed",
-								FALSE, FALSE,
-								(GdaThreadWrapperCallback) meta_changed_cb,
-								bcnc);
-
                         break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -633,7 +737,7 @@ clear_dsn_info (BrowserConnection *bcnc)
 }
 
 static void
-fav_changed_cb (G_GNUC_UNUSED BrowserFavorites *bfav, BrowserConnection *bcnc)
+fav_changed_cb (G_GNUC_UNUSED ToolsFavorites *bfav, BrowserConnection *bcnc)
 {
 	g_signal_emit (bcnc, browser_connection_signals [FAV_CHANGED], 0);
 }
@@ -696,8 +800,12 @@ browser_connection_dispose (GObject *object)
 			g_slist_foreach (bcnc->priv->p_mstruct_list, (GFunc) g_object_unref, NULL);
 			g_slist_free (bcnc->priv->p_mstruct_list);
 		}
+#if GLIB_CHECK_VERSION(2,31,7)
+		g_mutex_clear (&bcnc->priv->mstruct_mutex);
+#else
 		if (bcnc->priv->p_mstruct_mutex)
 			g_mutex_free (bcnc->priv->p_mstruct_mutex);
+#endif
 
 		if (bcnc->priv->cnc)
 			g_object_unref (bcnc->priv->cnc);
@@ -780,7 +888,7 @@ check_for_wrapper_result (BrowserConnection *bcnc)
 				/* nothing to do */
 			}
 			else {
-				g_mutex_lock (bcnc->priv->p_mstruct_mutex);
+				MUTEX_LOCK (bcnc);
 				
 				if (bcnc->priv->c_mstruct) {
 					GdaMetaStruct *old_mstruct;
@@ -802,7 +910,7 @@ check_for_wrapper_result (BrowserConnection *bcnc)
 #endif
 					g_signal_emit (bcnc, browser_connection_signals [META_CHANGED], 0, bcnc->priv->mstruct);
 				}
-				g_mutex_unlock (bcnc->priv->p_mstruct_mutex);
+				MUTEX_UNLOCK (bcnc);
 			}
 			break;
 		}
@@ -1150,14 +1258,14 @@ browser_connection_rollback (BrowserConnection *bcnc, GError **error)
  *
  * Get @bcnc's favorites handler
  *
- * Returns: (transfer none): the #BrowserFavorites used by @bcnc
+ * Returns: (transfer none): the #ToolsFavorites used by @bcnc
  */
-BrowserFavorites *
+ToolsFavorites *
 browser_connection_get_favorites (BrowserConnection *bcnc)
 {
 	g_return_val_if_fail (BROWSER_IS_CONNECTION (bcnc), NULL);
 	if (!bcnc->priv->bfav && !BROWSER_IS_VIRTUAL_CONNECTION (bcnc)) {
-		bcnc->priv->bfav = browser_favorites_new (gda_connection_get_meta_store (bcnc->priv->cnc));
+		bcnc->priv->bfav = tools_favorites_new (gda_connection_get_meta_store (bcnc->priv->cnc));
 		g_signal_connect (bcnc->priv->bfav, "favorites-changed",
 				  G_CALLBACK (fav_changed_cb), bcnc);
 	}
@@ -1257,7 +1365,7 @@ wrapper_statement_execute (StmtExecData *data, GError **error)
 		else {
 			g_warning (_("Execution reported an undefined error, please report error to "
 				     "http://bugzilla.gnome.org/ for the \"libgda\" product"));
-			g_set_error (error, 0, 0, _("No detail"));
+			g_set_error (error, 0, 0, "%s", _("No detail"));
 		}
 	}
 	return obj ? obj : (gpointer) 0x01;
@@ -1652,7 +1760,7 @@ meta_store_addons_init (BrowserConnection *bcnc, GError **error)
 
 	if (!bcnc->priv->cnc) {
 		g_set_error (error, 0, 0,
-			     _("Connection not yet opened"));
+			     "%s", _("Connection not yet opened"));
 		return FALSE;
 	}
 	store = gda_connection_get_meta_store (bcnc->priv->cnc);
@@ -2354,7 +2462,7 @@ wrapper_ldap_search (LdapSearchData *data, GError **error)
 					 data->filter, data->attributes, data->scope);
 	if (!model) {
 		g_set_error (error, 0, 0,
-			     _("Could not execute LDAP search"));
+			     "%s", _("Could not execute LDAP search"));
 		return (gpointer) 0x01;
 	}
 	else {
